=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-07-12 04:38:52 +0000
+++ bzrlib/branch.py	2005-07-12 07:17:28 +0000
@@ -627,7 +627,19 @@
             
         assert r.revision_id == revision_id
         return r
-        
+
+    def get_revisions(self, revision_ids. pb=None):
+        """Return the Revision object for a set of named revisions"""
+        from bzrlib.revision import Revision
+        from bzrlib.xml import unpack_xml
+
+        self.lock_read()
+        try:
+            for f in self.revision_store.get(revision_ids, pb=pb):
+                yield unpack_xml(Revision, f)
+        finally:
+            self.unlock()
+            
 
     def get_revision_sha1(self, revision_id):
         """Hash the stored value of a revision, and return it."""
@@ -649,7 +661,24 @@
         from bzrlib.inventory import Inventory
         from bzrlib.xml import unpack_xml
 
-        return unpack_xml(Inventory, self.inventory_store[inventory_id])
+        self.lock_read()
+        try:
+            return unpack_xml(Inventory, self.inventory_store[inventory_id])
+        finally:
+            self.unlock()
+
+    def get_inventories(self, inventory_ids, pb=None):
+        """Get Inventory objects by id
+        """
+        from bzrlib.inventory import Inventory
+        from bzrlib.xml import unpack_xml
+
+        self.lock_read()
+        try:
+            for f in self.inventory_store.get(inventory_ids, pb=pb):
+                yield unpack_xml(Inventory, f)
+        finally:
+            self.unlock()
             
 
     def get_inventory_sha1(self, inventory_id):
@@ -849,24 +878,25 @@
         if hasattr(other.revision_store, "prefetch"):
             other.revision_store.prefetch(revision_ids)
         if hasattr(other.inventory_store, "prefetch"):
-            inventory_ids = [other.get_revision(r).inventory_id
-                             for r in revision_ids]
-            other.inventory_store.prefetch(inventory_ids)
+            other.inventory_store.prefetch(revision_ids)
                 
-        revisions = []
+        revisions = self.get_revisions(revision_ids, pb=pb)
+        inventories = self.get_inventories(revision_ids, pb=pb)
         needed_texts = set()
         i = 0
-        for rev_id in revision_ids:
+
+        for rev, inv in zip(revisions, inventories):
             i += 1
             pb.update('fetching revision', i, len(revision_ids))
-            rev = other.get_revision(rev_id)
-            revisions.append(rev)
-            inv = other.get_inventory(str(rev.inventory_id))
+            text_ids = []
             for key, entry in inv.iter_entries():
                 if entry.text_id is None:
                     continue
-                if entry.text_id not in self.text_store:
-                    needed_texts.add(entry.text_id)
+                text_ids.append(entry.text_id)
+            has_ids = self.text_store.has(text_ids)
+            for has, text_id in zip(has_ids, text_ids):
+                if not has:
+                    needed_texts.add(text_id)
 
         pb.clear()
                     

=== modified file 'bzrlib/http_transport.py'
--- bzrlib/http_transport.py	2005-07-12 06:45:24 +0000
+++ bzrlib/http_transport.py	2005-07-12 07:17:28 +0000
@@ -76,7 +76,10 @@
         
 
 class HttpTransport(Transport):
-    """This is the transport agent for local filesystem access."""
+    """This is the transport agent for http:// access.
+    
+    TODO: Implement pipelined versions of all of the *_multi() functions.
+    """
 
     def __init__(self, base):
         """Set the base path where files will be stored."""
@@ -129,13 +132,6 @@
         except urllib2.URLError:
             return False
 
-    def has_multi(self, relpaths):
-        """Return True/False for each entry in relpaths
-
-        TODO: Implement HttpTransport.has_multi()
-        """
-        return super(HttpTransport, self).has_multi(relpaths)
-
     def get(self, relpath, decode=False):
         """Get the file at the given relative path.
 

=== modified file 'bzrlib/store.py'
--- bzrlib/store.py	2005-07-12 00:16:45 +0000
+++ bzrlib/store.py	2005-07-12 07:17:28 +0000
@@ -79,6 +79,23 @@
         for fileid, f in entries:
             self.add(fileid, f)
 
+    def has(self, fileids):
+        """Return True/False for each entry in fileids.
+
+        :param fileids: A List or generator yielding file ids.
+        :return: A generator or list returning True/False for each entry.
+        """
+        for fileid in fileids:
+            if fileid in self:
+                yield True
+            else:
+                yield False
+
+    def get(self, fileids, pb=None):
+        """Return a set of files, one for each requested entry."""
+        for fileid in fileids:
+            yield self[fileid]
+
     def copy_multi(self, other, ids):
         """Copy texts for ids from other into self.
 
@@ -234,7 +251,25 @@
         fn = self._relpath(fileid)
         return self._transport.has(fn)
 
-    # TODO: Guard against the same thing being stored twice, compressed and uncompresse
+    def has(self, fileids, pb=None):
+        """Return True/False for each entry in fileids.
+
+        :param fileids: A List or generator yielding file ids.
+        :return: A generator or list returning True/False for each entry.
+        """
+        relpaths = [self._relpath(fid) for fid in fileids]
+        return self._transport.has_multi(relpaths, pb=pb)
+
+    def get(self, fileids, pb=None):
+        """Return a set of files, one for each requested entry."""
+        rel_paths = [self._relpath(fid) for fid in fileids]
+        for f in self._transport.get_multi(rel_paths, pb=pb):
+            if hasattr(f, 'tell'):
+                yield gzip.GzipFile(mode='rb', fileobj=f)
+            else:
+                from cStringIO import StringIO
+                sio = StringIO(f.read())
+                yield gzip.GzipFile(mode='rb', fileobj=sio)
 
     def __iter__(self):
         # TODO: case-insensitive?

=== modified file 'bzrlib/transport.py'
--- bzrlib/transport.py	2005-07-12 06:16:50 +0000
+++ bzrlib/transport.py	2005-07-12 07:17:28 +0000
@@ -169,12 +169,12 @@
         """Does the target location exist?"""
         raise NotImplementedError
 
-    def has_multi(self, relpaths):
+    def has_multi(self, relpaths, pb=None):
         """Return True/False for each entry in relpaths"""
         total = self._get_total(relpaths)
         count = 0
         for relpath in relpaths:
-            self._update_pb(pb, 'get', count, total)
+            self._update_pb(pb, 'has', count, total)
             yield self.has(relpath)
             count += 1
 

